"""
Unit tests.
"""
__revision__ = "1.0.2"

from elatom import Config, Elatom, FeedReader, ProgressBar
import os
import time
import unittest

class TestConfig(unittest.TestCase):
    """
    Unit test class for Config.
    """
    _config = None
    _configfile = None
    _logfile = None

    def setUp(self):
        """
        The test runner will run that method prior to each test.
        """
        self._configfile = os.path.abspath("test_configfile.conf")
        self._logfile = os.path.abspath("test_logfile.log")
        if os.path.exists(self._configfile):
            os.unlink(self._configfile)
        if os.path.exists(self._logfile):
            os.unlink(self._logfile)
        self._config = Config(configfile=self._configfile,
                                logfile=self._logfile)

    def tearDown(self):
        """
        The test runner will invoke that method after each test.
        """
        if os.path.exists(self._configfile):
            os.unlink(self._configfile)
        if os.path.exists(self._logfile):
            os.unlink(self._logfile)

    def test_activate_feed(self):
        """
        Test Config.activate_feed()
        """
        self.test_add_feed()
        feed_name = "Yii Radio"
        self.assertTrue(feed_name in self._config.feeds(True))
        self._config.activate_feed(feed_name, None, False)
        self.assertTrue(feed_name not in self._config.feeds(True))

    def test_add_feed(self):
        """
        Test Config.add_feed()
        """
        found = len(self._config.feeds())
        self.assertEqual(found, 0)

        url = "\
http://feeds.feedburner.com/YiiRadiio-APodcastAboutTheYiiphpFramework"
        feed_name = "Yii Radio"
        self._config.add_feed(name=feed_name, url=url)
        found = len(self._config.feeds())
        self.assertEqual(found, 1)
        url = "http://feeds.feedburner.com/phpclassesblog"
        feed_name = "PHPClasses"
        self._config.add_feed(name=feed_name, url=url)
        found = len(self._config.feeds())
        self.assertEqual(found, 2)

    def test_delete_feed(self):
        """
        Test Config.delete_feed()
        """
        self.test_add_feed()
        url = "\
http://feeds.feedburner.com/YiiRadiio-APodcastAboutTheYiiphpFramework"
        feed_name = "Yii Radio"
        self._config.delete_feed(name=feed_name, url=url)
        found = len(self._config.feeds())
        self.assertEqual(found, 1, "Fist deletion")
        url = "http://feeds.feedburner.com/phpclassesblog"
        feed_name = "PHPClasses"
        self._config.delete_feed(name=feed_name, url=url)
        found = len(self._config.feeds())
        self.assertEqual(found, 0, "Second deletion")

    def test_download_directory(self):
        """
        Test Config.download_directory()
        """
        new_dir = 'download'
        self._config.download_directory(new_dir)
        found = self._config.download_directory()
        self.assertEqual(found, new_dir, "Save a new download directory")

    def test_memory(self):
        """
        Test Config.memory() and Config.is_downloaded()
        """
        url = "http://localhost/media.ogg"
        found = self._config.is_downloaded(url)
        self.assertFalse(found, "The url %s should be unknown." % url)
        self._config.memory(url)
        found = self._config.is_downloaded(url)
        self.assertTrue(found, "The url %s should be known." % url)
        self._config.memory(url)
        found = self._config.is_downloaded(url)
        self.assertTrue(found, "The url %s should be known." % url)

class TestElAtom(unittest.TestCase):
    """
    Main unit test class for Elatom.
    """

    @staticmethod
    def _feed_parser(text):
        """
        Use FeedReader to get the URL.
        """
        import xml.etree.ElementTree as etree
        tree = etree.fromstring(text)
        return FeedReader.parse_feed(tree)

    @staticmethod
    def _feed_parser_file(filename):
        """
        Run _feed_parser() on a file of fixtures/.
        """
        f = open(os.path.join('fixtures', filename))
        text = f.read()
        f.close()
        return TestElAtom._feed_parser(text)

    def test_feed_parser(self):
        """
        Test FeedReader.
        """
        files = {
                'atom-1.0.atom':
                    (
                        'http://www.example.org/myaudiofile.mp3',
                        'http://www.example.org/myaudiofile.torrent'
                    ),
                'atom-1.0-translated.atom':
                    (
                        'ftp://www.example.org/en/bloggingguidelines.pdf',
                        'ftp://www.example.org/en/bloggingguidelines.pdf',
                        'ftp://www.example.org/en/bloggingguidelines.pdf',
                    ),
                'la_marche_de_lhistoire.xml':
                    (
                        'http://rf.proxycast.org/m/media/273073201426.mp3?c=culture&p=La+marche+de+l%27histoire_11739&l3=20120319&l4=&media_url=http%3A%2F%2Fmedia.radiofrance-podcast.net%2Fpodcast09%2F11739-19.03.2012-ITEMA_20353598-0.mp3',
                        'http://rf.proxycast.org/m/media/273073201426.mp3?c=culture&p=La+marche+de+l%27histoire_11739&l3=20120319&l4=&media_url=http%3A%2F%2Fmedia.radiofrance-podcast.net%2Fpodcast09%2F11739-16.03.2012-ITEMA_20352938-0.mp3',
                        'http://rf.proxycast.org/m/media/273073201426.mp3?c=culture&p=La+marche+de+l%27histoire_11739&l3=20120319&l4=&media_url=http%3A%2F%2Fmedia.radiofrance-podcast.net%2Fpodcast09%2F11739-15.03.2012-ITEMA_20352642-0.mp3',
                        'http://rf.proxycast.org/m/media/273073201426.mp3?c=culture&p=La+marche+de+l%27histoire_11739&l3=20120319&l4=&media_url=http%3A%2F%2Fmedia.radiofrance-podcast.net%2Fpodcast09%2F11739-14.03.2012-ITEMA_20352353-0.mp3',
                        'http://rf.proxycast.org/m/media/273073201426.mp3?c=culture&p=La+marche+de+l%27histoire_11739&l3=20120319&l4=&media_url=http%3A%2F%2Fmedia.radiofrance-podcast.net%2Fpodcast09%2F11739-13.03.2012-ITEMA_20352063-0.mp3',
                        'http://rf.proxycast.org/m/media/273073201426.mp3?c=culture&p=La+marche+de+l%27histoire_11739&l3=20120319&l4=&media_url=http%3A%2F%2Fmedia.radiofrance-podcast.net%2Fpodcast09%2F11739-12.03.2012-ITEMA_20351782-0.mp3',
                    ),
                'rss-0.92.rss':
                    (
                        'http://www.scripting.com/mp3s/weatherReportDicksPicsVol7.mp3',
                        'http://www.scripting.com/mp3s/theOtherOne.mp3',
                        'http://www.scripting.com/mp3s/franklinsTower.mp3',
                        'http://www.scripting.com/mp3s/youWinAgain.mp3',
                        'http://www.scripting.com/mp3s/weatherReportSuite.mp3',
                        'http://www.scripting.com/mp3s/darkStar.mp3',
                        'http://www.scripting.com/mp3s/usBlues.mp3',
                        'http://www.scripting.com/mp3s/tennesseeJed.mp3',
                        'http://www.scripting.com/mp3s/darkStarWharfRat.mp3',
                        'http://www.scripting.com/mp3s/friendOfTheDevil.mp3',
                        'http://www.scripting.com/mp3s/uncleJohnsBand.mp3',
                        'http://www.scripting.com/mp3s/meAndMyUncle.mp3',
                        'http://www.scripting.com/mp3s/truckin.mp3',
                        'http://www.scripting.com/mp3s/touchOfGrey.mp3',
                        'http://www.scripting.com/mp3s/usBlues.mp3',
                    ),
                'rss-2.0.rss':
                   (
                       'http://www.example.org/myaudiofile.mp3',
                   )
        }
        for filename in files:
            results = TestElAtom._feed_parser_file(filename)
            self.assertTrue(None != results);
            self.assertEqual(len(results['files']), len(files[filename]))
            n = 0
            for url in files[filename]:
                self.assertEqual(results['files'][n]['url'], files[filename][n])
    
    def test_feed_parser_guid(self):
        """
        Test FeedReader, the extraction of guid.
        """
        files = {
                'atom-1.0.atom':
                    (
                        'http://www.example.org/myaudiofile.mp3',
                        'http://www.example.org/myaudiofile.torrent'
                    ),
                'atom-1.0-translated.atom':
                    (
                        'ftp://www.example.org/en/bloggingguidelines.pdf',
                        'ftp://www.example.org/en/bloggingguidelines.pdf',
                        'ftp://www.example.org/en/bloggingguidelines.pdf',
                    ),
                'la_marche_de_lhistoire.xml':
                   (
                       'http://media.radiofrance-podcast.net/podcast09/11739-19.03.2012-ITEMA_20353598-0.mp3',
                       'http://media.radiofrance-podcast.net/podcast09/11739-16.03.2012-ITEMA_20352938-0.mp3',
                       'http://media.radiofrance-podcast.net/podcast09/11739-15.03.2012-ITEMA_20352642-0.mp3',
                       'http://media.radiofrance-podcast.net/podcast09/11739-14.03.2012-ITEMA_20352353-0.mp3',
                       'http://media.radiofrance-podcast.net/podcast09/11739-13.03.2012-ITEMA_20352063-0.mp3',
                       'http://media.radiofrance-podcast.net/podcast09/11739-12.03.2012-ITEMA_20351782-0.mp3',
                   ),
                'rss-2.0.rss':
                   (
                       'http://www.example.org/entries/1',
                   )
        }
        for filename in files:
            results = TestElAtom._feed_parser_file(filename)
            self.assertTrue(None != results);
            self.assertEqual(len(results['files']), len(files[filename]))
            n = 0
            for url in files[filename]:
                expected = files[filename][n]
                actual = results['files'][n]['id']
                self.assertEqual(actual, expected, "Wrong extraction for the file %s. %s != %s" % (filename, actual, expected))

    def test_format_date(self):
        """
        Test Elatom.format_date()
        """
        dates = {
                # ATOM
                '2005-07-15T12:00:00Z':'20050715',
                '2011-08-17T00:11:08Z':'20110817',
                # RSS
                'Fri, 15 Jul 2005 00:00:00 -0500':'20050715',
                'Tue, 18 Oct 2011 11:40:00 +0200':'20111018',
                }
        for given in dates:
            expected = dates[given]
            found = Elatom.format_date(given)
            self.assertEqual(found, expected)

    def test_format_title(self):
        """
        Test Elatom.format_title.
        """
        titles = {
                'PHPTV - Podcast':'PHPTV',
                'Yii Radio - a PHP podcast focusing on the Yii framework': 
                        'YiiRadioaPHPfocusing',
                'Episode #52':'Episode52',
                'Is PHP 5.4 really coming?':'IsPHP54reallycoming',
                }
        for given in titles:
            expected = titles[given]
            found = Elatom.format_title(given)
            self.assertEqual(found, expected)

class TestProgressBar(unittest.TestCase):
    """
    Test ProgressBar.
    """
    def test_progress_bar(self):
        """
        Test ProgressBar.
        """
        tests = {
                0: '[>                 0%                  ]',
                10: '[===>             10%                  ]',
                50: '[=================50%                  ]',
                75: '[=================75%=======>          ]',
                66.66: '[=================67%====>             ]',
                100: '[================100%==================]'
        }
        limit = 100
        width = 40
        prog = ProgressBar(0, limit, width)
        for i in tests:
            prog.set_value(i)
            found = str(prog)
            expected = tests[i]
            self.assertEqual(found, expected)
            self.assertEqual(len(found), width)

class TestDownload(unittest.TestCase):
    """
    Test Download.
    """
    def xtest_download(self):
        """
        Test Download.
        """
        from elatom import Download
        temp_filename = '/tmp/test.tar.gz'
        download = Download('http://localhost/distfiles.php/' + 
                'ati-driver-installer-10-12-x86.x86_64.run', temp_filename)
        download.start()
        print()
        print("Waiting")
        print()
        while download.size == None:
            time.sleep(0.25)
        print(download.size)
        while download.is_ended == False:
            print(download.downloaded / download.size * 100.0, "    \r",
                    end="")
            time.sleep(0.25)
        print("Finished !   \n", end="")
        self.assertTrue(os.path.exists(temp_filename))
        os.unlink(temp_filename)

if __name__ == '__main__':
    unittest.main()
